@@ -14,7 +14,6 @@
 struct xfs_trans;
 struct xfs_mount;
 struct xfs_inode_log_item;
-struct xfs_dir_ops;
 
 /*
  * These are not actually used, they are only for userspace build
@@ -33,6 +32,8 @@ struct xfs_dir_ops;
  */
 struct inode {
 	mode_t		i_mode;
+	uint32_t	i_uid;
+	uint32_t	i_gid;
 	uint32_t	i_nlink;
 	xfs_dev_t	i_rdev;		/* This actually holds xfs_dev_t */
 	unsigned long	i_state;	/* Not actually used in userspace */
@@ -43,6 +44,23 @@ struct inode {
 	struct timespec	i_ctime;
 };
 
+static inline uint32_t i_uid_read(struct inode *inode)
+{
+	return inode->i_uid;
+}
+static inline uint32_t i_gid_read(struct inode *inode)
+{
+	return inode->i_gid;
+}
+static inline void i_uid_write(struct inode *inode, uint32_t uid)
+{
+	inode->i_uid = uid;
+}
+static inline void i_gid_write(struct inode *inode, uint32_t gid)
+{
+	inode->i_gid = gid;
+}
+
 typedef struct xfs_inode {
 	struct cache_node	i_node;
 	struct xfs_mount	*i_mount;	/* fs mount struct ptr */
@@ -60,7 +78,6 @@ typedef struct xfs_inode {
 	unsigned int		i_cformat;	/* format of cow fork */
 
 	xfs_fsize_t		i_size;		/* in-memory size */
-	const struct xfs_dir_ops *d_ops;	/* directory ops vector */
 	struct xfs_ifork_ops	*i_fork_ops;	/* fork verifiers */
 	struct inode		i_vnode;
 } xfs_inode_t;
